# Data Anonymization for Machine Learning

## Overview
With the enforcement of a series of data privacy protection regulations such as GDPR, data sharing has become a tricky problem. This tool is intends to facilitate data sharing from customer by anonymize private and sensitive data while maintaining the data utility for machine learning. 
Two parts are included in this tool:
+ Data anonymization
  Anonymize the original dataset with specific parameters. It accepts CSV data as input, and supports different anonymization methods(remove, pseudonym, or differential privacy).
+ Data utility evaluation after anonymization 
  Evaluate the data utiltiy after data anonymization. Input orignal dataset and anonymized dataset and one utility evluation report will be generated with several indicators. This part can also faciliate user to choose the suitable anonymization methods.

## Quickstart
+ Install python >= 3.6.0 from [python.org](https://www.python.org/)
+ Download source code: run `git clone https://github.wdf.sap.corp/ICN-Nanjing-Projects/Data-Anonymization.git` or download manually;
+ Install: run `python setup.py install`;
+ Download demo data: [adult.txt](https://github.wdf.sap.corp/ICN-Nanjing-Projects/Data-Anonymization/files/13887/adult.txt) and put it under folder *Data-Anonymization*;
+ Anonymize: run `data-anonymize adult.txt`;
+ Evaluate: run `data-evaluate adult.txt adult_a.csv --class-label sex,marital-status,race`;
+ Review evaluation report: *report.html* ([Detail Information](https://github.wdf.sap.corp/ICN-Nanjing-Projects/Data-Anonymization/wiki/User-Guide-of-Data-Anonymization/#general-concepts)) under folder *adult*.

## Download and Installation
+ Install Python 
  Ensure your python >=3.6.0, you can download it from [python.org](https://www.python.org/)
+ After clone the project, install it as follows
  ```
  python setup.py install
  ```
  The Data-Anonymization project provides two commands: `data-anonymize` (for anonymization) and `data-evaluate` (for evaluation). Run `data-anonymize -h` and `data-evaluate -h` to read parameters description.

+ Help of `data-anonymize`
  Run `data-anonymize -h` to show its usage manuals.
  ```
  usage: data-anonymize [-h] [--pseudonym LIST] [--delete LIST]
                        [--na-values LIST] [-o FILE] [--no-header] [-e FLOAT]
                        [--category LIST] [--retain LIST]
                        file

  Anonymize one dataset by Differential Privacy

  positional arguments:
    file                 set path of the csv to be anonymized

  general arguments:
    -h, --help           show this help message and exit
    --pseudonym LIST     set candidate columns separated by comma, which are
                        taken as sensitive keys, e.g. phone, email, etc. will
                        do pseudonym on these columns. Only works on string
                        column.
    --delete LIST        set columns separated by comma, which will be deleted
                        when anonymization.
    --na-values LIST     set additional values to recognize as NA/NaN; (default
                        null values are from pandas.read_csv)
    -o, --output FILE    set the file name of output anonymized dataset (default
                        is input file name with suffix '_a')
    --no-header          indicate there is no header in a csv file, and will
                        take [#0, #1, #2, ...] as header. (default: the tool
                        will try to detect and take actions)

  advanced arguments:
    -e, --epsilon FLOAT  set epsilon for differential privacy (default 0.1)
    --category LIST      set categorical columns separated by comma.
    --retain LIST        set columns to retain the values
  ```


+ Help of `data-evaluate`

  ```
  usage: data-evaluate [-h] [--na-values LIST] [-o FILE] [-t TEST]
                      [--class-label LIST]
                      source target

  Evaluate anonymized data set and compare with source data set.

  positional arguments:
    source              set file path of source (raw) dataset to be compare with
                        anonymized dataset, support csv files
    target              set file path of target (anonymized) dataset to evaluate

  general arguments:
    -h, --help          show this help message and exit
    --na-values LIST    set additional values to recognize as NA/NaN; (default
                        null values are from pandas.read_csv)
    -o, --output FILE   set output path for evaluation report; (default is
                        "report.html" under current work directory)

  advanced arguments:
    -t, --test TEST     set test dataset for classification or regression task;
                        (default take 20 percent from source dataset)
    --class-label LIST  set column name as class label for classification or
                        regression task; supports one or multiple columns
                        (separated by comma)
  ```
  
## How to obtain support
If you encounter an issue, your can open an issue in GitHub

## License
Copyright (c) 2019 SAP SE or an SAP affiliate company. All rights reserved. This file is licensed under the Apache Software License, v.2 except as noted otherwise in the [LICENSE file](https://github.com/zy-yan/DA4ML/blob/master/LICENSE).
